#region

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Net;
using System.Text.RegularExpressions;
using System.Windows.Forms;
using System.Xml.Serialization;
using EveMapSettings;
using libEveMapNG.Data.Dynamic;
using libEveMapNG.Data.Static;
using libEveMapNG.RestAPI;

#endregion

namespace EVEMapNG.SubForms
{
    public partial class ManageCynoArrays : Form
    {
        public ManageCynoArrays()
        {
            InitializeComponent();
        }

        private void ManageCynoArrays_Load(object sender, EventArgs e)
        {
            UpdateListBox();
        }

        private void btnAdd_Click(object sender, EventArgs e)
        {
            CynoBeacon cynoF = new CynoBeacon();
            try
            {
                cynoF.System = SolarSystem.GetByName(ssbSystem.Text);
            }
            catch
            {
                MessageBox.Show("That system doesn't exist or has been mistyped.");
                ssbSystem.Text = String.Empty;
                return;
            }
            if (DynamicStore.Instance.CynoBeacons.ContainsKey(cynoF.System))
            {
                MessageBox.Show("That system already has a beacon.");
                return;
            }

            DynamicStore.Instance.CynoBeacons.Add(cynoF.System, cynoF);
            UpdateListBox();
        }

        private void btnRemove_Click(object sender, EventArgs e)
        {
            if (lvCynoBeacons.SelectedIndices.Count != 1)
                return;
            DynamicStore.Instance.CynoBeacons.Remove(((CynoBeacon) lvCynoBeacons.SelectedItems[0].Tag).System);
            UpdateListBox();
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            DynamicStore.WriteToFile();
            Close();
        }

        private void UpdateListBox()
        {
            lvCynoBeacons.Items.Clear();
            ListViewItem lvi;
            foreach (KeyValuePair<SolarSystem, CynoBeacon> cynoArray in DynamicStore.Instance.CynoBeacons)
            {
                lvi = new ListViewItem(cynoArray.Value.System.Name)
                          {
                              Tag = cynoArray.Value
                          };
                lvi.SubItems.Add(cynoArray.Value.Note);
                lvCynoBeacons.Items.Add(lvi);
            }
        }

        private void btnImportCyno_Click(object sender, EventArgs e)
        {
            if (!Clipboard.ContainsData(DataFormats.Html))
            {
                MessageBox.Show("Please copy the cyno array list in the wiki before you push button.", Text,
                                MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }
            string clipboardContents = (string) Clipboard.GetData(DataFormats.Html);
            clipboardContents = clipboardContents.Replace("\r", "");
            clipboardContents = clipboardContents.Replace("\n", "");

            Regex rGetCynoArrays =
                new Regex(
                    @"<td>(?<region>[a-zA-Z0-9\- ]+)</td><td>(?<system>[a-zA-Z0-9\-]+)</td><td>(?<note>[a-zA-Z0-9\- ]*)</td>",
                    RegexOptions.ExplicitCapture | RegexOptions.IgnoreCase);
            DynamicStore.Instance.CynoBeacons.Clear();
            foreach (Match m in rGetCynoArrays.Matches(clipboardContents))
            {
                CynoBeacon cyno = new CynoBeacon();
                try
                {
                    cyno.System = SolarSystem.GetByName(m.Groups["system"].Value);
                    cyno.Note = m.Groups["note"].Value;
                }
                catch (Exception exception)
                {
                    MessageBox.Show(
                        String.Format(
                            "Failed to add array, {0}. Please fix the wiki.",
                            exception.Message), "Invalid Solar System", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    continue;
                }
                DynamicStore.Instance.CynoBeacons.Add(cyno.System, cyno);
            }
            UpdateListBox();
        }

        private void btnClear_Click(object sender, EventArgs e)
        {
            DialogResult result = MessageBox.Show(this,
                                                  "Are you sure you want to clear the list?  All cyno arrays will be removed.",
                                                  "Clear Cyno Arrays",
                                                  MessageBoxButtons.OKCancel, MessageBoxIcon.Warning,
                                                  MessageBoxDefaultButton.Button2);
            if (result == DialogResult.OK)
            {
                DynamicStore.Instance.CynoBeacons.Clear();
                UpdateListBox();
            }
        }

        private void lnkJumpbridgeWikiPage_LinkClicked_1(object sender, LinkLabelLinkClickedEventArgs e)
        {
            Process.Start("http://wiki.goonfleet.com/index.php?title=Cyno_Arrays");
        }

        private void btnRestApiImport_Click(object sender, EventArgs e)
        {
            //Cleanup some things and prep the GUI for importing
            Cursor regularCursor = Cursor;
            Cursor = Cursors.WaitCursor;
            Application.DoEvents();

            DynamicStore.Instance.CynoBeacons.Clear();

            //Get the XML from somewhere
            WebClient myClient = new WebClient();
            Stream apiXML = myClient.OpenRead(UserSettings.Instance.RestCynoBeaconUrl);

            //Deserialize it
            ApiCynoList myBeacons = (ApiCynoList) new XmlSerializer(typeof (ApiCynoList)).Deserialize(apiXML);

            //Import it
            CynoBeacon cyno;
            DynamicStore.Instance.CynoBeacons.Clear();
            foreach (ApiCyno beacon in myBeacons.Cynos)
            {
                if (beacon.SystemID == 0)
                    continue;
                cyno = new CynoBeacon
                           {
                               System = SolarSystem.GetByID(beacon.SystemID),
                               Note = beacon.Notes
                           };
                DynamicStore.Instance.CynoBeacons.Add(cyno.System, cyno);
            }

            //Update the GUI
            UpdateListBox();
            Cursor = regularCursor;
        }
    }
}