﻿using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;
using libEveMapNG.Data.Static;
using libEveMapNG.Renderer.Text;

namespace EveMapControls.Controls
{
    public partial class JumpPreview : UserControl
    {
        private float _distance;
        private float _isotopes;
        private SolarSystem _system;

        public JumpPreview()
        {
            InitializeComponent();
            lblIsotopes.Text = "";
            lblDistance.Text = "";
        }

        public SolarSystem SolarSystem
        {
            get { return _system; }
            set
            {
                _system = value;
                lblSystem.Text = _system.Name;
            }
        }

        public int SystemWidth { get; set; }

        public float Distance
        {
            set
            {
                _distance = value;
                lblDistance.Text = "Light Years: " + _distance;
            }
            get { return _distance; }
        }

        public float Isotopes
        {
            set
            {
                _isotopes = value;
                lblIsotopes.Text = "Isotopes: " + _isotopes;
            }
        }

        protected override void OnPaint(PaintEventArgs paintEventArgs)
        {
            //We need a graphics handle to this control to draw to it.
            Graphics g = CreateGraphics();
            g.SmoothingMode = SmoothingMode.HighQuality;

            Pen myPen = SolarSystemColorSchemes.SecurityScheme.GetSystemPen(_system, 6);

            lblSecurity.Text = "(" + Math.Round(_system.Security, 2) + ")";
            lblSecurity.ForeColor = myPen.Color;

            SolidBrush myBrush = new SolidBrush(myPen.Color);
            g.FillEllipse(myBrush, 5, 5, 22, 22);

            //We're done drawing if these thigns are empty
            //No need to show the lines and such
            if (String.IsNullOrEmpty(lblIsotopes.Text) || String.IsNullOrEmpty(lblDistance.Text))
                return;

            myPen.StartCap = LineCap.Flat;
            myPen.EndCap = LineCap.ArrowAnchor;

            g.DrawLine(myPen, 16, 15, 16, 80);
        }
    }
}