﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using libEveMapNG.Data.Dynamic;
using libEveMapNG.Data.Static;

namespace EveMapControls.MainFormTabs
{
    public partial class Details : UserControl
    {
        private SolarSystem curSystem;

        public Details()
        {
            InitializeComponent();
        }

        public SolarSystem CurrentSystem
        {
            set
            {
                curSystem = value;
                UpdateDetails();
            }
        }

        private void UpdateDetails()
        {
            int constellationId = curSystem.Constellation.ID;
            //get data
            var qryKills = (
                               from kill in DynamicStore.Instance.ApiData.MapKills
                               join sys in StaticStore.Systems on kill.Value.SolarSystemId equals sys.Value.ID
                               where sys.Value.ConstellationID == constellationId
                               select new
                                          {
                                              SystemId = sys.Value.ID,
                                              ShipKillCount = kill.Value.ShipKills,
                                              PodKillCount = kill.Value.PodKills,
                                              FactionKillCount = kill.Value.FactionKills
                                          });
            var qryJumps = (
                               from jump in DynamicStore.Instance.ApiData.MapJumps
                               join sys in StaticStore.Systems on jump.Value.SolarSystemId equals sys.Value.ID
                               where sys.Value.ConstellationID == constellationId
                               select new
                                          {
                                              SystemId = sys.Value.ID,
                                              JumpCount = jump.Value.ShipJumps
                                          });
            var systemKills = (from k in qryKills where k.SystemId == curSystem.ID select k);
            var systemJump = (from j in qryJumps where j.SystemId == curSystem.ID select j);
            //set basics
            txtSystem.Text = curSystem.Name;
            txtRegion.Text = curSystem.Region.Name;
            txtConstellation.Text = curSystem.Constellation.Name;
            txtSecurity.Text = string.Format("{0} ({1})", Math.Round(curSystem.Security, 2), curSystem.SecurityClass);
            //system only
            var systemSov = (from sov in DynamicStore.Instance.ApiData.MapSovereignty
                             join alliance in DynamicStore.Instance.ApiData.Alliances on sov.Value.AllianceId equals
                                 alliance.Key
                             where sov.Key == curSystem.ID
                             select new
                                        {
                                            alliance.Value.Name,
                                            alliance.Value.Ticker
                                        }).SingleOrDefault();
            if (systemSov != null)
                txtSystemSov.Text = string.Format("{0} ({1})", systemSov.Name, systemSov.Ticker);
            else
                txtSystemSov.Text = "Unclaimed";
            if (systemJump.Count() > 0)
                txtSystemJumps.Text = systemJump.First().JumpCount.ToString();
            else
                txtSystemJumps.Text = "0";
            if (systemKills.Count() > 0)
            {
                txtSystemKills.Text = string.Format("PC {0} Pod {1} NPC {2}",
                                                    systemKills.First().ShipKillCount,
                                                    systemKills.First().PodKillCount,
                                                    systemKills.First().FactionKillCount);
            }
            else
                txtSystemKills.Text = "All Silent";
            //Constellation
            if (qryKills.Count() > 0)
            {
                txtConstellationKills.Text = string.Format("PC {0} Pod {1} NPC {2}",
                                                           (from k in qryKills select k.ShipKillCount).Sum(),
                                                           (from k in qryKills select k.PodKillCount).Sum(),
                                                           (from k in qryKills select k.FactionKillCount).Sum());
            }
            else
                txtConstellationKills.Text = "All Silent";
            if (qryJumps.Count() > 0)
                txtConstellationJumps.Text = (from j in qryJumps select j.JumpCount).Sum().ToString();
            else
                txtConstellationJumps.Text = "0";

            //tree
            treeAdditionalDetails.Nodes.Clear();
            IEnumerable<Station> allStations = (
                                                   from s in DynamicStore.Instance.ApiData.ConquerableStations
                                                   where s.Value.SolarSystemId == curSystem.ID
                                                   select new Station
                                                              {
                                                                  Agents = new List<Agent>(),
                                                                  Cloning = false,
                                                                  Faction = 0,
                                                                  Factory = false,
                                                                  ID = 0,
                                                                  Name = s.Value.Name,
                                                                  Owner =
                                                                      (from c in
                                                                           DynamicStore.Instance.ApiData.Corporations
                                                                       where c.Value.Id == s.Value.CorporationId
                                                                       select c.Value.Name).First(),
                                                                  Refine = 0.0d,
                                                                  Services = new List<string>(),
                                                                  SolarSystemID = (ushort) s.Value.SolarSystemId
                                                              });
            if (curSystem.Stations != null)
                allStations = allStations.Union(from s in curSystem.Stations select s);
            //tree stations
            treeAdditionalDetails.Nodes.Add(GetStationNode(allStations));
            //tree planets
            treeAdditionalDetails.Nodes.Add(GetPlanetNode(curSystem));
            //tree moons
            treeAdditionalDetails.Nodes.Add(GetMoonNode(curSystem));
            //tree belts
            treeAdditionalDetails.Nodes.Add(GetBeltNode(curSystem));
            //tree ice belts
            treeAdditionalDetails.Nodes.Add(GetIceFieldNode(curSystem));
            //tree agents            
            IEnumerable<Agent> agents = from s in allStations
                                        where s.Agents != null
                                        from a in s.Agents
                                        select a;


            treeAdditionalDetails.Nodes.Add(GetAgentNode(agents, true, allStations));
        }

        private static TreeNode GetBeltNode(SolarSystem system)
        {
            if (system.Planets == null)
                return new TreeNode("Belts (0)");

            int count = 0;
            foreach (Planet planet in system.Planets)
            {
                count += planet.Belts;
            }

            TreeNode root = new TreeNode(string.Format("Belts ({0})", count));

            Planet[] planets = system.Planets.OrderBy(p => p.Number).ToArray();
            foreach (Planet p in planets)
            {
                if (p.Belts > 0)
                {
                    for (int b = 1; b <= p.Belts; b++)
                    {
                        root.Nodes.Add(string.Format("Belt {0}-{1}", p.Number, b));
                    }
                }
            }
            return root;
        }

        private static TreeNode GetIceFieldNode(SolarSystem system)
        {
            if (system.Planets == null)
                return new TreeNode("Ice Fields (0)");

            int count = 0;
            foreach (Planet planet in system.Planets)
            {
                count += planet.IceFields;
            }

            TreeNode root = new TreeNode(string.Format("Ice Fields ({0})", count));

            Planet[] planets = system.Planets.OrderBy(p => p.Number).ToArray();
            foreach (Planet p in planets)
            {
                if (p.IceFields > 0)
                {
                    for (int i = 1; i <= p.IceFields; i++)
                        root.Nodes.Add(string.Format("Ice Field {0}-{1}", p.Number, i));
                }
            }
            return root;
        }

        private static TreeNode GetMoonNode(SolarSystem system)
        {
            if (system.Planets == null)
                return new TreeNode("Moons (0)");

            int count = 0;
            foreach (Planet planet in system.Planets)
            {
                count += planet.Moons;
            }
            TreeNode root = new TreeNode(string.Format("Moons ({0})", count));
            Planet[] Planets = system.Planets.OrderBy(p => p.Number).ToArray();
            foreach (Planet p in Planets)
            {
                if (p.Moons > 0)
                {
                    for (int m = 1; m <= p.Moons; m++)
                        root.Nodes.Add(string.Format("Moon {0}-{1}", p.Number, m));
                }
            }
            return root;
        }

        private static TreeNode GetPlanetNode(SolarSystem system)
        {
            if (system.Planets == null)
                return new TreeNode("Planets (0)");
            TreeNode root = new TreeNode(string.Format("Planets ({0})", system.Planets.Count()));
            TreeNode child;
            Planet[] Planets = system.Planets.OrderBy(p => p.Number).ToArray();
            foreach (Planet p in Planets)
            {
                child = root.Nodes.Add(string.Format("Planet {0}", p.Number));
                if (p.Belts > 0)
                    child.Nodes.Add(string.Format("Belts ({0})", p.Belts));
                if (p.IceFields > 0)
                    child.Nodes.Add(string.Format("Ice Fields ({0})", p.IceFields));
                if (p.Moons > 0)
                    child.Nodes.Add(string.Format("Moons ({0})", p.Moons));
            }
            return root;
        }

        private static TreeNode GetStationNode(IEnumerable<Station> allStations)
        {
            TreeNode root = new TreeNode(string.Format("Stations ({0})", allStations.Count()));
            TreeNode child;
            foreach (Station stat in allStations)
            {
                child = CreateSationNode(stat);
                if (stat.Agents != null && stat.Agents.Count > 0)
                    child.Nodes.Add(GetAgentNode(stat.Agents, false, null));
                root.Nodes.Add(child);
            }
            return root;
        }

        private static TreeNode CreateSationNode(Station stat)
        {
            TreeNode root = new TreeNode(stat.Name);
            if (!string.IsNullOrEmpty(stat.Owner))
                root.Nodes.Add(string.Format("Owner = {0}", stat.Owner));
            if (stat.Cloning)
                root.Nodes.Add("Cloning");
            if (stat.Factory)
                root.Nodes.Add("Factory");
            if (stat.Refine > 0)
                root.Nodes.Add(string.Format("Refine ({0})", Math.Round(stat.Refine, 2)));
            return root;
        }

        private static TreeNode GetAgentNode(IEnumerable<Agent> allAgents, bool showStation,
                                             IEnumerable<Station> allStations)
        {
            TreeNode root = new TreeNode(string.Format("Agents ({0})", allAgents.Count()));

            TreeNode child;
            foreach (Agent agent in allAgents)
            {
                child = root.Nodes.Add(agent.Name);
                child.Nodes.Add(string.Format("Level {0}", agent.Level));
                child.Nodes.Add(string.Format("Quality {0}", agent.Quality));
                child.Nodes.Add(string.Format("Division {0}", agent.Division));
                child.Nodes.Add(agent.Type);
                if (showStation)
                {
                    Station stat = (
                                       from s in allStations
                                       where s.ID == agent.StationID
                                       select s).First();
                    child.Nodes.Add(CreateSationNode(stat));
                }
            }
            return root;
        }
    }
}