﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using EveMapControls.Controls;
using EveMapSettings;
using libEveMapNG.Data.Dynamic;
using libEveMapNG.Data.Static;
using libEveMapNG.Utility;

namespace EveMapControls.MainFormTabs
{
    public partial class ManualJumpPlanner : UserControl
    {
        private readonly List<SolarSystem> Route = new List<SolarSystem>();

        public ManualJumpPlanner()
        {
            InitializeComponent();
            cboShip.Items.RemoveAt(0);
            foreach (KeyValuePair<string, CapitalShips.Ship> ship in CapitalShips.Ships)
                cboShip.Items.Add(ship.Value.Name);
        }

        public MapViewport Viewport { get; set; }

        public void UpdateSystemsInJumpRange(SolarSystem system)
        {
            Viewport.FilterSystems = true;
            if (cboJDC.SelectedIndex < 0)
                return;
            if (Viewport.SelectedSystem != null)
            {
                Viewport.UnfilteredSystems = SolarSystem.GetSystemsInRange(system,
                                                                           cboJDC.SelectedIndex,
                                                                           CapitalShips.Ships[
                                                                               (string)cboShip.SelectedItem]);
                Viewport.Invalidate();
            }
        }

        private void UpdateFilteredSystems()
        {
            if (Viewport.SelectedSystem != null)
            {
                Viewport.UnfilteredSystems = SolarSystem.GetSystemsInRange(Viewport.SelectedSystem,
                                                                           cboJDC.SelectedIndex,
                                                                           CapitalShips.Ships[
                                                                               (string)cboShip.SelectedItem]);
            }
        }

        public void SaveState()
        {
            if (ssbSystem.SelectedSystem != null)
                UserSettings.Instance.AppState.MJPState.JumpDestinationSystemId = ssbSystem.SelectedSystem.ID;

            UserSettings.Instance.AppState.MJPState.JDC = cboJDC.SelectedIndex;
            UserSettings.Instance.AppState.MJPState.JFC = cboJFC.SelectedIndex;
            UserSettings.Instance.AppState.MJPState.JF = cboJF.SelectedIndex;
            UserSettings.Instance.AppState.MJPState.Ship = cboShip.SelectedIndex;
        }

        public void RestoreState()
        {
            if (UserSettings.Instance.AppState.MJPState == null)
                UserSettings.Instance.AppState.LoadDefaultSettings();

            cboJDC.SelectedIndex = UserSettings.Instance.AppState.MJPState.JDC;
            cboJFC.SelectedIndex = UserSettings.Instance.AppState.MJPState.JFC;
            cboJF.SelectedIndex = UserSettings.Instance.AppState.MJPState.JF;
            cboShip.SelectedIndex = UserSettings.Instance.AppState.MJPState.Ship;
        }

        private ShipProfile CapitalShipProfile;
        private void btnAdd_Click(object sender, EventArgs e)
        {
            if (ssbSystem.SelectedSystem == null)
                return;

            int jdc, jf, jfc;
            if (!int.TryParse((String)cboJDC.SelectedItem, out jdc) ||
                !int.TryParse((String)cboJF.SelectedItem, out jf) ||
                !int.TryParse((String)cboJFC.SelectedItem, out jfc))
            {
                MessageBox.Show("Please select valid JDC/JF/JFC Values.", "Invalid Parameters");
                return;
            }

            CapitalShips.Ship ship;
            if (!CapitalShips.Ships.TryGetValue(cboShip.Text, out ship))
            {
                MessageBox.Show("Please select a valid ship.", "Invalid Ship");
                return;
            }

            CapitalShipProfile = new ShipProfile
                                                 {
                                                     JDC = jdc,
                                                     Ship = ship,
                                                     JF = jf,
                                                     JFC = jfc
                                                 };
            Route.Add(ssbSystem.SelectedSystem);
            UpdatePreview(CapitalShipProfile);
        }

        private void UpdatePreview(ShipProfile capitalShipProfile)
        {
            foreach (Control con in Controls.Find("JumpPreview", true))
            {
                Controls.Remove(con);
            }

            for (int i = 0; i < Route.Count; i++)
            {
                if (Route.Count > 1 && (i + 1) > Route.Count)
                {
                    continue;
                }

                JumpPreview myPreview = new JumpPreview();

                myPreview.Name = "JumpPreview";
                myPreview.Location = new Point(5, (grpJumpCapSetup.Location.Y + grpJumpCapSetup.Height) + 10 + (i * 80));

                if (Route.Count != 1)
                {
                    if (i + 1 != Route.Count)
                    {
                        myPreview.Distance = JumpDrive.GetDistance(Route[i], Route[i + 1]);
                        myPreview.Isotopes = JumpDrive.GetFuel(myPreview.Distance, capitalShipProfile);
                    }
                }

                myPreview.SolarSystem = Route[i];
                myPreview.Click += ClickHandler;

                myPreview.btnMoveUp.Click +=MoveUpClickHandler;
                myPreview.btnMoveDown.Click += MoveDownClickHandler;
                myPreview.DoubleClick += DoubleClickHandler;
                Controls.Add(myPreview);
            }
        }

        private  void ClickHandler(object sender, EventArgs e)
        {
            JumpPreview myPreview = (JumpPreview)sender;

        }

        private  void DoubleClickHandler(object sender, EventArgs e)
        {
            JumpPreview myPreview = (JumpPreview) sender;
            Route.Remove(myPreview.SolarSystem);

            UpdatePreview(CapitalShipProfile);
        }

        private void MoveUpClickHandler(object sender, EventArgs e)
        {
            JumpPreview myPreview = (JumpPreview)((Button)sender).Parent;

            int i = Route.IndexOf(myPreview.SolarSystem);
            int n = Route.IndexOf(myPreview.SolarSystem) - 1;

            var tmp = Route[n];
            Route[n] = Route[i];
            Route[i] = tmp;

            UpdatePreview(CapitalShipProfile);
        }

        private void MoveDownClickHandler(object sender, EventArgs e)
        {
            JumpPreview myPreview = (JumpPreview)((Button)sender).Parent;

            int i = Route.IndexOf(myPreview.SolarSystem);
            int n = Route.IndexOf(myPreview.SolarSystem) + 1;

            var tmp = Route[n];
            Route[n] = Route[i];
            Route[i] = tmp;

            UpdatePreview(CapitalShipProfile);
        }
    }
}