using System.Drawing;

namespace EveMapControls
{
    internal static class Utility
    {
        // converting screen to/from EVESystem.FlatX/Y coordinates
        // Scale: from flat xy to screen xy
        // Unscale: from screen xy to flat xy
        // note: adjustX/Y are set by AdjustScrollBars()

        public static PointF ScalePoint(PointF point, Point adjust, float scale)
        {
            point.X = (point.X - adjust.X)*scale;
            point.Y = (point.Y - adjust.Y)*scale;
            //return Point.Round(point);
            return point;
        }

        public static PointF UnscalePoint(PointF point, Point adjust, float scale)
        {
            point.X = point.X/scale + adjust.X;
            point.Y = point.Y/scale + adjust.Y;
            //return Point.Round(point);
            return point;
        }

        public static bool IsLineVisible(PointF p1, PointF p2, RectangleF viewRect, Point adjust, float mapScale)
        {
            // first, we check whether either of the points are inside the viewing rectangle.  if not,
            // we check whether the line intersects any of the sides of the viewing rectangle.  if so,
            // it is visible inside the rectangle

            if (viewRect.Contains(p1))
                return true;
            if (viewRect.Contains(p2))
                return true;

            if (LinesIntersect(p1, p2, new PointF(viewRect.Left, viewRect.Top), new PointF(viewRect.Right, viewRect.Top)))
                return true;
            if (LinesIntersect(p1, p2, new PointF(viewRect.Left, viewRect.Top),
                               new PointF(viewRect.Left, viewRect.Bottom))) return true;
            if (LinesIntersect(p1, p2, new PointF(viewRect.Right, viewRect.Bottom),
                               new PointF(viewRect.Left, viewRect.Bottom))) return true;
            if (LinesIntersect(p1, p2, new PointF(viewRect.Right, viewRect.Bottom),
                               new PointF(viewRect.Right, viewRect.Top))) return true;
            return false;
        }

        private static bool LinesIntersect(PointF a1, PointF a2, PointF b1, PointF b2)
        {
            // to accurately test whether two line segments AB and CD intersect, we test whether point A and B are on opposite
            // sides of line CD, and whether point C and D are on opposite sides of AB.  we do this by testing whether
            // you turn in the same direction when turning between the points (thanks, "Numerical Recipes in C"!)

            return (TurnDirection(a1, a2, b1)*TurnDirection(a1, a2, b2) <= 0) &&
                   (TurnDirection(b1, b2, a1)*TurnDirection(b1, b2, a2) <= 0);
        }

        private static int TurnDirection(PointF a, PointF b, PointF c)
            // return code: 1 counter-clockwise, -1 clockwise, 0 collinear with no turn
        {
            float dABX = (b.X - a.X),
                  dABY = (b.Y - a.Y),
                  dACX = (c.X - a.X),
                  dACY = (c.Y - a.Y);

            if (dABY*dACX < dACY*dABX) return 1;
            if (dABY*dACX > dACY*dABX) return -1;

            // special cases with collinear points AKA line right at edge of view rectangle
            if (dABX*dACX < 0 || dABY*dACY < 0) return -1;
            if (dABX*dABX + dABY*dABY >= dACX*dACX + dACY*dACY) return 0;
            return 1;
        }

        public static bool IsAreaVisible(RectangleF testArea, RectangleF viewRect, Point adjust, float mapScale,
                                         bool prescaled)
        {
            if (!prescaled)
            {
                PointF topLeft = ScalePoint(new PointF(testArea.Left, testArea.Top), adjust, mapScale);
                PointF bottomRight = ScalePoint(new PointF(testArea.Right, testArea.Bottom), adjust, mapScale);
                testArea = RectangleF.FromLTRB(topLeft.X, topLeft.Y, bottomRight.X, bottomRight.Y);
            }

            return viewRect.IntersectsWith(testArea);
        }
    }
}