﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using libEveMapNG.Data;
using libEveMapNG.Data.Static;
using libEveMapNG.Utility;

namespace Test
{
    internal class Program
    {
        public static void Main()
        {
            Loader.LoadAllData();
            var route = new List<Node>();

            //Throw all systems into a node collection
            List<Node> allNodes = new List<Node>();
            foreach (KeyValuePair<int, SolarSystem> system in StaticStore.Systems)
            {
                if (system.Value.Security > 0.45f)
                    continue;
                var newNode = new Node
                                  {
                                      System = system.Value,
                                      ConnectedNodes = new List<Node>()
                                  };
                allNodes.Add(newNode);
            }

            //Find out what nodes are in range of the parent node
            foreach (Node fromNode in allNodes)
            {
                SolarSystem fromSystem = fromNode.System;
                foreach (Node toNode in allNodes)
                {
                    if (toNode == fromNode)
                        continue;

                    SolarSystem toSystem = toNode.System;
                    if (JumpDrive.GetDistance(toSystem, fromSystem) <= 14.625f)
                        fromNode.ConnectedNodes.Add(toNode);
                }
            }

            Node startNode = (from node in allNodes
                              where node.System == SolarSystem.GetByName("J-LPX7")
                              select node).Single();

            Node goalNode = (from node in allNodes
                             where node.System == SolarSystem.GetByName("KZFV-4")
                             select node).Single();

            Stopwatch myWatch = new Stopwatch();
            myWatch.Start();
            // A* pathing algorithm
            var openNodes = new List<Node>();
            startNode.GDistanceFromLastNode = 0;
            GetH(goalNode, startNode);
            startNode.FSumGH = startNode.HMinDistanceToTravel;
            startNode.RoutingParentNode = startNode;

            openNodes.Add(startNode);
            while (openNodes.Count > 0)
            {
                openNodes.Sort();
                Node curNode = openNodes[0];
                openNodes.Remove(curNode);

                curNode.RoutingClosed = true;
                if (curNode == goalNode) break;

                foreach (Node subNode in curNode.ConnectedNodes)
                {
                    if (!subNode.RoutingClosed)
                    {
                        if (!openNodes.Contains(subNode))
                        {
                            subNode.RoutingParentNode = curNode;
                            openNodes.Add(subNode);
                        }
                        else
                        {
                            if (GetG(curNode) < subNode.GDistanceFromLastNode)
                            {
                                subNode.RoutingParentNode = curNode;
                            }
                        }

                        CalculateFGH(subNode.RoutingParentNode, goalNode);
                    }
                }
            }

            // add all the route _nodes to an arraylist by backtracking until we're at the start
            if (goalNode.RoutingParentNode != null)
            {
                var backtrack = new List<Node>();
                Node trackNode = goalNode.RoutingParentNode;

                while (trackNode != startNode)
                {
                    backtrack.Add(trackNode);
                    trackNode = trackNode.RoutingParentNode;
                }

                // add all intermediate jump systems to JumpRoute.Jumps in the right order
                if (backtrack.Count > 0)
                {
                    for (int i = backtrack.Count - 1; i >= 0; i--)
                    {
                        Node jumpNode = backtrack[i];
                        route.Add(jumpNode);
                    }
                }
            }
            myWatch.Stop();
            foreach (Node list in route)
            {
                Console.WriteLine(list.System.Name);
            }
            Console.WriteLine("Time Taken by A*: {0}", myWatch.Elapsed);

            Console.Read();
        }

        public static void CalculateFGH(Node parentNode, Node goalNode)
        {
            parentNode.GDistanceFromLastNode = GetG(parentNode);
            parentNode.FSumGH = parentNode.GDistanceFromLastNode + parentNode.HMinDistanceToTravel;
            parentNode.Steps = parentNode.RoutingParentNode.Steps + 1;
        }

        public static void GetH(Node goalNode, Node currentNode)
        {
            currentNode.HMinDistanceToTravel = JumpDrive.GetDistance(currentNode.System, goalNode.System);
            currentNode.StepsLeft = currentNode.HMinDistanceToTravel/14.625f;
        }

        public static double GetG(Node parentNode)
        {
            return parentNode.GDistanceFromLastNode +
                   JumpDrive.GetDistance(parentNode.RoutingParentNode.System, parentNode.System);
        }

        #region Nested type: Node

        public class Node : IComparable
        {
            public List<Node> ConnectedNodes;
            public double FSumGH;
            public double GDistanceFromLastNode;
            public double HMinDistanceToTravel;
            public bool RoutingClosed;
            public Node RoutingParentNode;
            public int Steps;
            public double StepsLeft;
            public SolarSystem System;

            #region IComparable Members

            public int CompareTo(object obj)
            {
                var node = (Node) obj;
                double cmp1 = FSumGH;
                double cmp2 = node.FSumGH;
                return cmp1.CompareTo(cmp2);
            }

            #endregion
        }

        #endregion
    }
}