#region

using System;
using System.IO;
using System.Reflection;

#endregion

namespace Eve_Class_Library
{
    [Serializable]
    public class EVEData
    {
        private const string FILENAME = "empyrean.eve";
        [NonSerialized] private static readonly object _padLock = new object();
        [NonSerialized] private static EVEData _singleton;
        public EVEAgentList AgentList = new EVEAgentList();
        public EVEAgentMetaList AgentMetaList = new EVEAgentMetaList();
        public EVEFactionList FactionList = new EVEFactionList();
        public EVEGateList GateList = new EVEGateList();
        public EVERegionList RegionList = new EVERegionList();
        public EVEStationList StationList = new EVEStationList();
        public EVEStationMetaList StationMetaList = new EVEStationMetaList();

        public static EVEData Instance
        {
            get
            {
                lock (_padLock)
                {
                    if (_singleton == null)
                    {
                        try
                        {
                            Directory.SetCurrentDirectory(
                                Path.GetDirectoryName(Assembly.GetExecutingAssembly().GetModules()[0].FullyQualifiedName));
                            Console.WriteLine(Directory.GetCurrentDirectory());
                            _singleton = ReadFromFile();
                        }
                        catch
                        {
                            throw new FileNotFoundException(
                                "Fatal Error:  EVE Map Data file not found.\nProgram will now close.");
                        }
                    }
                    return _singleton;
                }
            }
        }

        public void Clear()
        {
            RegionList = new EVERegionList();
            FactionList = new EVEFactionList();
            StationList = new EVEStationList();
            StationMetaList = new EVEStationMetaList();
            GateList = new EVEGateList();
            AgentList = new EVEAgentList();
            AgentMetaList = new EVEAgentMetaList();
        }

        public static void WriteToFile()
        {
            EVESerializer.DataSerializer.WriteToFile(FILENAME, _singleton);
        }

        public static EVEData ReadFromFile()
        {
            return EVESerializer.DataSerializer.ReadFromFile<EVEData>(FILENAME);
        }
    }
}