#region

using System;
using System.Collections.Generic;

#endregion

namespace Eve_Class_Library
{
    [Serializable]
    public class EVEFactionList : List<EVEFaction>
    {
        public EVEFaction GetByID(int id)
        {
            foreach (EVEFaction curFaction in this)
            {
                if (curFaction.ID == id) return curFaction;
            }

            return null;
        }

        public EVECorpNPC GetCorpNPCByID(int id)
        {
            foreach (EVEFaction curFaction in this)
            {
                EVECorpNPC retCorpNPC = curFaction.GetByID(id);

                if (retCorpNPC != null) return retCorpNPC;
            }

            return null;
        }

        public EVECorpNPC GetCorpNPCByName(string name)
        {
            foreach (EVEFaction curFaction in this)
            {
                EVECorpNPC retCorpNPC = curFaction.GetByName(name);

                if (retCorpNPC != null) return retCorpNPC;
            }

            return null;
        }
    }

    [Serializable]
    public class EVEFaction
    {
        private readonly List<EVECorpNPC> corpNPCs = new List<EVECorpNPC>();
        public int ID;
        public string Name;

        public IList<EVECorpNPC> CorpNPCs
        {
            get { return corpNPCs; }
        }

        public EVECorpNPC GetByID(int id)
        {
            foreach (EVECorpNPC curCorpNPC in corpNPCs)
            {
                if (curCorpNPC.ID == id) return curCorpNPC;
            }

            return null;
        }

        public EVECorpNPC GetByName(string name)
        {
            foreach (EVECorpNPC curCorpNPC in corpNPCs)
            {
                if (curCorpNPC.Name == name) return curCorpNPC;
            }

            return null;
        }
    }

    [Serializable]
    public class EVECorpNPC
    {
        public int ID;
        public string Name;
        public EVEFaction ParentFaction;
    }

    [Serializable]
    public class EVEAgentList : List<EVEAgent>
    {
        public EVEAgent GetByID(int id)
        {
            foreach (EVEAgent curAgent in this)
            {
                if (curAgent.ID == id) return curAgent;
            }

            return null;
        }
    }

    [Serializable]
    public class EVEAgent
    {
        private readonly List<EVEResearchSkill> researchSkills = new List<EVEResearchSkill>();
        public EVEAgentType AgentType;
        public EVECorpNPC CorpNPC;
        public EVEAgentDivision Division;
        public int ID;
        public byte Level;
        public bool Locator;
        public string Name;

        public EVEStation ParentStation;
        public sbyte Quality;

        public IList<EVEResearchSkill> ResearchSkills
        {
            get { return researchSkills; }
        }
    }

    [Serializable]
    public class EVEAgentMetaList
    {
        private readonly List<EVEAgentDivision> agentDivisions = new List<EVEAgentDivision>();
        private readonly List<EVEAgentType> agentTypes = new List<EVEAgentType>();
        private readonly List<EVEResearchSkill> researchSkills = new List<EVEResearchSkill>();

        public IList<EVEAgentDivision> AgentDivisions
        {
            get { return agentDivisions; }
        }

        public IList<EVEAgentType> AgentTypes
        {
            get { return agentTypes; }
        }

        public IList<EVEResearchSkill> ResearchSkills
        {
            get { return researchSkills; }
        }

        public EVEAgentDivision GetAgentDivisionByID(int id)
        {
            foreach (EVEAgentDivision curAgentDivision in agentDivisions)
            {
                if (curAgentDivision.ID == id) return curAgentDivision;
            }

            return null;
        }

        public EVEAgentType GetAgentTypeByID(int id)
        {
            foreach (EVEAgentType curAgentType in agentTypes)
            {
                if (curAgentType.ID == id) return curAgentType;
            }

            return null;
        }

        public EVEResearchSkill GetResearchSkillByID(int id)
        {
            foreach (EVEResearchSkill curResearchSkill in researchSkills)
            {
                if (curResearchSkill.ID == id) return curResearchSkill;
            }

            return null;
        }
    }

    [Serializable]
    public class EVEAgentDivision
    {
        public int ID;
        public string Name;
    }

    [Serializable]
    public class EVEAgentType
    {
        public int ID;
        public string Name;
    }

    [Serializable]
    public class EVEResearchSkill
    {
        public int ID;
        public string Name;
    }
}