#region

using System;
using System.Collections.Generic;
using System.Drawing;

#endregion

namespace Eve_Class_Library
{
    public enum RatType
    {
        None,
        Gurista,
        Angel,
        Serpentis,
        Blood,
        Sansha,
        Drone
    }

    [Serializable]
    public class EVERegionList : List<EVERegion>
    {
        [NonSerialized] public Rectangle FlatArea;
        [NonSerialized] public bool FlatDataChanged;

        public EVERegion GetRegionByID(int id)
        {
            foreach (EVERegion curRegion in this)
            {
                if (curRegion.ID == id) return curRegion;
            }

            return null;
        }

        public EVEConstellation GetConstellationByID(int id)
        {
            foreach (EVERegion curRegion in this)
            {
                EVEConstellation retConst = curRegion.GetConstellationByID(id);
                if (retConst != null) return retConst;
            }

            return null;
        }

        public EVESystem GetSystemByID(int id)
        {
            foreach (EVERegion curRegion in this)
            {
                EVESystem retSystem = curRegion.GetSystemByID(id);
                if (retSystem != null) return retSystem;
            }

            return null;
        }

        public EVESystem GetSystemByName(string name)
        {
            foreach (EVERegion curRegion in this)
            {
                EVESystem retSystem = curRegion.GetSystemByName(name);
                if (retSystem != null) return retSystem;
            }

            return null;
        }

        public void FillFlatBoundaries()
        {
            Point FlatMin = new Point(int.MaxValue, int.MaxValue);
            Point FlatMax = new Point(int.MinValue, int.MinValue);

            foreach (EVERegion curRegion in this)
            {
                curRegion.FillFlatBoundaries();

                if (curRegion.FlatArea.Left < FlatMin.X) FlatMin.X = curRegion.FlatArea.Left;
                if (curRegion.FlatArea.Right > FlatMax.X) FlatMax.X = curRegion.FlatArea.Right;
                if (curRegion.FlatArea.Top < FlatMin.Y) FlatMin.Y = curRegion.FlatArea.Top;
                if (curRegion.FlatArea.Bottom > FlatMax.Y) FlatMax.Y = curRegion.FlatArea.Bottom;
            }

            FlatArea = Rectangle.FromLTRB(FlatMin.X, FlatMin.Y, FlatMax.X, FlatMax.Y);
        }
    }

    [Serializable]
    public class EVERegion : List<EVEConstellation>
    {
        public Color Color = Color.Black;
        [NonSerialized] public Rectangle FlatArea;
        public int ID;
        public string Name;
        public EVEFaction ParentFaction;

        public RatType Rats
        {
            get
            {
                switch (Name)
                {
                    case "Cache":
                    case "Curse":
                    case "Detorid":
                    case "Feythabolis":
                    case "Great Wildlands":
                    case "Heimatar":
                    case "Immensea":
                    case "Insmother":
                    case "Metropolis":
                    case "Molden Heath":
                    case "Omist":
                    case "Scalding Pass":
                    case "Tenerifis":
                    case "Wicked Creek":
                    case "Impass":
                        return RatType.Angel;
                    case "Cobalt Edge":
                    case "Etherium Reach":
                    case "Malpais":
                    case "Oasa":
                    case "Perrigen Falls":
                    case "The Kalevale Expanse":
                    case "The Spire":
                        return RatType.Drone;
                    case "Catch":
                    case "Esoteria":
                    case "Paragon Soul":
                    case "Providence":
                    case "Stain":
                        return RatType.Sansha;
                    case "Branch":
                    case "Deklein":
                    case "Geminate":
                    case "Lonetrek":
                    case "Tenal":
                    case "The Citadel":
                    case "The Forge":
                    case "Tribute":
                    case "Vale of the Silent":
                    case "Venal":
                        return RatType.Gurista;
                    case "Delve":
                    case "Period Basis":
                    case "Querious":
                    case "Tash-Murkon":
                    case "Kador":
                    case "Khanid":
                    case "Kor-Azor":
                    case "Aridia":
                    case "Derelik":
                    case "Devoid":
                    case "Domain":
                    case "Genesis":
                    case "The Bleak Lands":
                        return RatType.Blood;
                    case "Cloud Ring":
                    case "Essence":
                    case "Everyshore":
                    case "Fade":
                    case "Fountain":
                    case "Outer Ring":
                    case "Placid":
                    case "Sinq Laison":
                    case "Solitude":
                    case "Syndicate":
                    case "Verge Vendor":
                    case "Pure Blind":
                        return RatType.Serpentis;
                    default:
                        return RatType.None;
                }
            }
        }

        public EVEConstellation GetConstellationByID(int id)
        {
            foreach (EVEConstellation curConstellation in this)
            {
                if (curConstellation.ID == id) return curConstellation;
            }

            return null;
        }

        public EVESystem GetSystemByID(int id)
        {
            foreach (EVEConstellation curConstellation in this)
            {
                EVESystem retSystem = curConstellation.GetSystemByID(id);
                if (retSystem != null) return retSystem;
            }

            return null;
        }

        public EVESystem GetSystemByName(string name)
        {
            foreach (EVEConstellation curConstellation in this)
            {
                EVESystem retSystem = curConstellation.GetSystemByName(name);
                if (retSystem != null) return retSystem;
            }

            return null;
        }

        public void FillFlatBoundaries()
        {
            Point FlatMin = new Point(int.MaxValue, int.MaxValue);
            Point FlatMax = new Point(int.MinValue, int.MinValue);

            foreach (EVEConstellation constellation in this)
            {
                constellation.FillFlatBoundaries();

                if (constellation.FlatArea.Left < FlatMin.X) FlatMin.X = constellation.FlatArea.Left;
                if (constellation.FlatArea.Right > FlatMax.X) FlatMax.X = constellation.FlatArea.Right;
                if (constellation.FlatArea.Top < FlatMin.Y) FlatMin.Y = constellation.FlatArea.Top;
                if (constellation.FlatArea.Bottom > FlatMax.Y) FlatMax.Y = constellation.FlatArea.Bottom;
            }

            FlatArea = Rectangle.FromLTRB(FlatMin.X, FlatMin.Y, FlatMax.X, FlatMax.Y);
        }
    }

    [Serializable]
    public class EVEConstellation : List<EVESystem>
    {
        [NonSerialized] public Rectangle FlatArea;
        public int ID;
        public string Name;
        public EVERegion ParentRegion;

        public Color Color
        {
            get { return ParentRegion.Color; }
        }

        public EVESystem GetSystemByID(int id)
        {
            foreach (EVESystem curSystem in this)
            {
                if (curSystem.ID == id) return curSystem;
            }

            return null;
        }

        public EVESystem GetSystemByName(string name)
        {
            foreach (EVESystem curSystem in this)
            {
                if (curSystem.Name == name) return curSystem;
            }

            return null;
        }

        public void FillFlatBoundaries()
        {
            Point FlatMin = new Point(int.MaxValue, int.MaxValue);
            Point FlatMax = new Point(int.MinValue, int.MinValue);

            foreach (EVESystem system in this)
            {
                if (system.FlatCoords.X < FlatMin.X) FlatMin.X = system.FlatCoords.X;
                if (system.FlatCoords.X > FlatMax.X) FlatMax.X = system.FlatCoords.X;
                if (system.FlatCoords.Y < FlatMin.Y) FlatMin.Y = system.FlatCoords.Y;
                if (system.FlatCoords.Y > FlatMax.Y) FlatMax.Y = system.FlatCoords.Y;
            }

            FlatArea = Rectangle.FromLTRB(FlatMin.X, FlatMin.Y, FlatMax.X, FlatMax.Y);
        }
    }

    public enum RelativePosition
    {
        Top,
        TopLeft,
        TopRight,
        CenterLeft,
        CenterRight,
        Bottom,
        BottomLeft,
        BottomRight,
        Center,
        None
    }

    [Serializable]
    public class EVESystem
    {
        private static readonly Dictionary<int, string> RomanNumerals = new Dictionary<int, string>
                                                                            {
                                                                                {1, "I"},
                                                                                {2, "II"},
                                                                                {3, "III"},
                                                                                {4, "IV"},
                                                                                {5, "V"},
                                                                                {6, "VI"},
                                                                                {7, "VII"},
                                                                                {8, "VIII"},
                                                                                {9, "IX"},
                                                                                {10, "X"},
                                                                                {11, "XI"},
                                                                                {12, "XII"},
                                                                                {13, "XIII"},
                                                                                {14, "XIV"},
                                                                                {15, "XV"},
                                                                                {16, "XVI"},
                                                                                {17, "XVII"},
                                                                                {18, "XVIII"},
                                                                                {19, "XIX"},
                                                                                {20, "XX"},
                                                                                {21, "XXI"},
                                                                                {22, "XXII"},
                                                                                {23, "XXIII"},
                                                                                {24, "XXIV"},
                                                                                {25, "XXV"},
                                                                                {26, "XXVI"},
                                                                                {27, "XXVII"},
                                                                                {28, "XXVIII"},
                                                                                {29, "XXIX"},
                                                                                {30, "XXX"},
                                                                                {31, "XXXI"},
                                                                                {32, "XXXII"},
                                                                                {33, "XXXIII"}
                                                                            };

        private readonly List<EVEGate> gates = new List<EVEGate>();
        private readonly List<EVEPlanet> planets = new List<EVEPlanet>();
        private readonly List<EVEStation> stations = new List<EVEStation>();

        [NonSerialized] public bool FilterPassed;
        [NonSerialized] public bool Filtered;
        public Point FlatCoords = Point.Empty;
        public RelativePosition FlatLabel;
        public int ID;
        public EVEConstellation ParentConstellation;
        public float Security;
        public byte SecurityClassLetter, SecurityClassNumber;

        public float X, Y, Z;
        public string Name { get; set; }

        public IList<EVEGate> Gates
        {
            get { return gates; }
        }

        public IList<EVEStation> Stations
        {
            get { return stations; }
        }

        public IList<EVEPlanet> Planets
        {
            get { return planets; }
        }

        public IList<EVEAgent> Agents
        {
            get
            {
                var agents = new List<EVEAgent>();
                foreach (EVEStation sta in stations)
                    foreach (EVEAgent agent in sta)
                        agents.Add(agent);
                return agents;
            }
        }

        public Color Color
        {
            get { return ParentConstellation.Color; }
        }

        public EVERegion ParentRegion
        {
            get { return ParentConstellation.ParentRegion; }
        }

        public static IEnumerable<EVESystem> FindSystems(string nameFragment)
        {
            foreach (EVERegion region in EVEData.Instance.RegionList)
                foreach (EVEConstellation constellation in region)
                    foreach (EVESystem system in constellation)
                        if (system.Name.ToLower().StartsWith(nameFragment))
                            yield return system;
        }

        public static EVESystem FindExactSystem(string SystemName)
        {
            foreach (EVERegion region in EVEData.Instance.RegionList)
                foreach (EVEConstellation constellation in region)
                    foreach (EVESystem system in constellation)
                        if (system.Name.ToLower().Equals(SystemName))
                            return system;
            return null;
        }

        public static string GetObjectName(int planetNumber, int objectNumber, string objectName)
        {
            return string.Format("Planet {0} - {1} {2}", RomanNumerals[planetNumber], objectName,
                                 RomanNumerals[objectNumber]);
        }

        public string GetSystemObjectName(int planetNumber, int objectNumber, string objectName)
        {
            return string.Format("{0} {1} - {2} {3}", Name, RomanNumerals[planetNumber], objectName,
                                 RomanNumerals[objectNumber]);
        }

        public EVEPlanet GetPlanetByNumber(int n)
        {
            foreach (EVEPlanet curPlanet in planets)
                if (curPlanet.Number == n) return curPlanet;

            return null;
        }

        public double GetDistance(EVESystem target)
        {
            return Math.Sqrt(GetDistanceSquared(target));
        }

        public double GetDistanceSquared(EVESystem target)
        {
            double dx = X - target.X, dy = Y - target.Y, dz = Z - target.Z;
            return dx*dx + dy*dy + dz*dz;
        }
    }

    public enum EVEGateType
    {
        Undefined,
        Normal,
        InterConstellation,
        InterRegion
    }

    [Serializable]
    public class EVEGateList : List<EVEGate>
    {
        public void RemoveDoubles()
        {
            for (int i = Count - 1; i >= 0; i--)
            {
                EVEGate curGate = this[i];
                for (int j = Count - 1; j >= 0; j--)
                {
                    EVEGate destGate = this[j];
                    if (curGate.FromSystem == destGate.ToSystem &&
                        curGate.ToSystem == destGate.FromSystem)
                    {
                        RemoveAt(i);
                        break;
                    }
                }
            }
        }

        public void RemoveOrphans()
        {
            // remove items without parenttosystem or parentfromsystem set
            //gates.RemoveAll(delegate(EVEGate gate) { return (gate.ParentToSystem == null || gate.ParentFromSystem == null); });

            for (int i = Count - 1; i >= 0; i--)
            {
                EVEGate curGate = this[i];
                if (curGate.ToSystem == null || curGate.FromSystem == null)
                    RemoveAt(i);
            }
        }
    }

    [Serializable]
    public class EVEGate
    {
        public EVESystem FromSystem, ToSystem;
        public EVEGateType Type;
    }

    [Serializable]
    public class EVEPlanet
    {
        public byte AsteroidBeltCount;
        public byte IceFieldCount;
        public byte MoonCount;
        public byte Number;

        public EVESystem ParentSystem;
    }
}