using System;
using System.Collections.Generic;
using libEveMapNG.Data.Static;

namespace libEveMapNG.Data.Dynamic
{
    [Serializable]
    public class RouteData
    {
        public float ShipMass; //In millions of KG

        public RouteData()
        {
            AvoidSystems = new List<SolarSystem>();
            RoutingOption = GateRouteOption.None;
            WayPoints = new List<SolarSystem>();
            UseBridges = true;
        }

        public bool UseBridges { get; set; }
        public SolarSystem StartSystem { get; set; }
        public List<SolarSystem> WayPoints { get; set; }
        public List<SolarSystem> AvoidSystems { get; set; }
        public SolarSystem DestSystem { get; set; }
        public GateRouteOption RoutingOption { get; set; }
    }

    [Serializable]
    public enum GateRouteOption
    {
        AvoidNullSec,
        AvoidLowSec,
        AvoidHighSec,
        None
    }

    public static class EdgeWeights
    {
        public static int GateBridge = 1;
        public static int GateDefault = 1;
        public static int GateHigh = 10;

        public static int JumpBeacon = 1;
        public static int JumpBridge = 1;
        public static int JumpDefault = 1;
        public static int JumpHigh = 10;
        public static int JumpStation = 2;

        public static void SetGateWeights(int High, int Bridge, int Default)
        {
            GateHigh = High;
            GateBridge = Bridge;
            GateDefault = Default;
        }

        public static void SetJumpWeights(int High, int Default, int Bridge, int Beacon, int Station)
        {
            JumpHigh = High;
            JumpBridge = Bridge;
            JumpDefault = Default;
            JumpBeacon = Beacon;
            JumpStation = Station;
        }
    }
}