﻿using System;
using System.IO;
using System.Threading;
using libeveapi;
using libEveMapNG.Data.Dynamic;
using libEveMapNG.Data.Static;
using libEveMapNG.Routes;
using libEveMapNG.Utility;

namespace libEveMapNG.Data
{
    public static class Loader
    {
        private static void ThreadedLoader()
        {
            LoadStaticStore();
            CacheBounds();
            LoadDynamicStore();

            Thread myThread = new Thread(LoadApiResponseCache);
            myThread.Start();
            myThread = new Thread(CreateUniverseGraph);
            myThread.Start();
        }


        /// <summary>
        ///   Pre-build the bounds so the viewport runs faster on first load
        /// </summary>
        private static void CacheBounds()
        {
            foreach (Region region in StaticStore.Instance.Regions)
            {
                region.GetBounds();
                foreach (Constellation constellation in region.Constellations)
                {
                    constellation.GetBounds();
                }
            }
        }

        private static void LoadDynamicStore()
        {
            DynamicStore.ReadFromFile();
        }

        private static void CreateUniverseGraph()
        {
            DynamicStore.Instance.UniverseGraph = JumpRouter.BuildGraph();
        }

        private static void LoadStaticStore()
        {
            StaticStore.ReadFromFile();
        }

        private static void LoadApiResponseCache()
        {
            try
            {
                using (
                    FileStream myStream = new FileStream(Files.EVEMapDirectory + "\\ResponseCache.xml", FileMode.Open))
                {
                    ResponseCache.Load(myStream);
                }
            }
            catch (FileNotFoundException)
            {
                //ignore
            }
        }

        public static void LoadAllData()
        {
            string eveMapDirectory = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData),
                                                  "\\EVE-Map NG\\");
            if (!Directory.Exists(eveMapDirectory))
            {
                Directory.CreateDirectory(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) +
                                          "\\EVE-Map NG\\");
            }

            //Load all needed data
            ThreadedLoader();
        }
    }
}