using System;
using System.Collections.Generic;
using System.Drawing;

namespace libEveMapNG.Data.Static
{
    [Serializable]
    public class Constellation
    {
        public Color Color;

        #region Bounds and Caching

        private Rectangle _bounds;

        public Rectangle Bounds
        {
            get { return GetBounds(); }
        }

        public Rectangle GetBounds()
        {
            if (_bounds.Size == Size.Empty)
            {
                Point Min = new Point(int.MaxValue, int.MaxValue);
                Point Max = new Point(int.MinValue, int.MinValue);
                foreach (SolarSystem system in Systems)
                {
                    if (system.OrganicCoordinates.X < Min.X)
                        Min.X = system.OrganicCoordinates.X;
                    if (system.OrganicCoordinates.Y < Min.Y)
                        Min.Y = system.OrganicCoordinates.Y;
                    if (system.OrganicCoordinates.X > Max.X)
                        Max.X = system.OrganicCoordinates.X;
                    if (system.OrganicCoordinates.Y > Max.Y)
                        Max.Y = system.OrganicCoordinates.Y;
                }
                _bounds = new Rectangle(Min.X, Min.Y, Max.X - Min.X, Max.Y - Min.Y);
            }
            return _bounds;
        }

        #endregion

        public List<int> JumpAdjacentConstellations { get; set; }
        public int ID { get; set; }
        public string Name { get; set; }
        public int RegionID { get; set; }
        public List<SolarSystem> Systems { get; set; }

        public static Constellation GetByID(int ID)
        {
            return StaticStore.Constellations[ID];
        }

        public static bool operator ==(Constellation a, Constellation b)
        {
            // If both are null, or both are same instance, return true.
            if (ReferenceEquals(a, b))
                return true;
            // If one is null, but not both, return false.
            if (((object) a == null) || ((object) b == null))
                return false;
            return a.ID == b.ID;
        }

        public static bool operator !=(Constellation a, Constellation b)
        {
            return !(a == b);
        }

        public bool Equals(Constellation obj)
        {
            if (ReferenceEquals(null, obj))
                return false;
            if (ReferenceEquals(this, obj))
                return true;
            return obj.ID == ID;
        }

        public override bool Equals(object obj)
        {
            if (ReferenceEquals(null, obj))
                return false;
            if (ReferenceEquals(this, obj))
                return true;
            if (obj.GetType() != typeof (Constellation))
                return false;
            return Equals((Constellation) obj);
        }

        public override int GetHashCode()
        {
            return ID;
        }
    }
}