using System;
using System.Collections.Generic;
using System.Drawing;

namespace libEveMapNG.Data.Static
{
    [Serializable]
    public class Region
    {
        public int ID { get; set; }
        public string Name { get; set; }
        public List<Constellation> Constellations { get; set; }

        #region Bounds and Caching

        public Color Color;
        private Rectangle _bounds;

        public Rectangle Bounds
        {
            get
            {
                if (_bounds.Size != SizeF.Empty)
                    return _bounds;
                return GetBounds();
            }
        }

        public Rectangle GetBounds()
        {
            Point min = new Point(int.MaxValue, int.MaxValue);
            Point max = new Point(int.MinValue, int.MinValue);
            foreach (Constellation constellation in Constellations)
            {
                Rectangle bounds = constellation.Bounds;
                if (bounds.Left < min.X)
                    min.X = bounds.Left;
                if (bounds.Top < min.Y)
                    min.Y = bounds.Top;
                if ((bounds.Left + bounds.Width) > max.X)
                    max.X = (bounds.Left + bounds.Width);
                if ((bounds.Top + bounds.Height) > max.Y)
                    max.Y = (bounds.Top + bounds.Height);
            }
            _bounds = new Rectangle(min.X, min.Y, max.X - min.X, max.Y - min.Y);
            return _bounds;
        }

        #endregion

        public static Region GetByID(int id)
        {
            foreach (Region region in StaticStore.Instance.Regions)
            {
                if (region.ID == id)
                    return region;
            }
            return null;
        }

        public static bool operator ==(Region a, Region b)
        {
            // If both are null, or both are same instance, return true.
            if (ReferenceEquals(a, b))
                return true;
            // If one is null, but not both, return false.
            if (((object) a == null) || ((object) b == null))
                return false;
            return a.ID == b.ID;
        }

        public static bool operator !=(Region a, Region b)
        {
            return !(a == b);
        }

        private bool Equals(Region obj)
        {
            if (ReferenceEquals(null, obj))
                return false;
            if (ReferenceEquals(this, obj))
                return true;
            return obj.ID == ID;
        }

        public override bool Equals(object obj)
        {
            if (ReferenceEquals(null, obj))
                return false;
            if (ReferenceEquals(this, obj))
                return true;
            if (obj.GetType() != typeof (Region))
                return false;
            return Equals((Region) obj);
        }

        public override int GetHashCode()
        {
            return ID;
        }
    }
}