using System;
using System.Collections.Generic;
using System.IO;
using libeveapi;
using libEveMapNG.Data.Dynamic;
using libEveMapNG.Data.Static;
using libEveMapNG.EveAPI.Data;
using libEveMapNG.Utility;

namespace libEveMapNG.EveAPI
{
    [Serializable]
    public class EveAPIData
    {
        private const string responseCacheFilename = "\\ResponseCache.xml";
        private Dictionary<int, Alliance> alliances;
        private ApiInfo apiinfo;
        private Dictionary<int, Character> characters;
        private Dictionary<int, ConquerableStation> conquerables;
        private Dictionary<int, Corporation> corporations;
        private Dictionary<int, Kills> kills;
        private Dictionary<int, Jumps> mapJumps;
        private Dictionary<int, Sovereignty> mapSovereignty;
        private Dictionary<int, Starbase> starbases;

        #region Properties

        public DateTime ApiLastRefresh { get; set; }

        public ApiInfo ApiInfo
        {
            get
            {
                if (apiinfo != null)
                    return apiinfo;
                apiinfo = new ApiInfo();
                return apiinfo;
            }
        }

        public Dictionary<int, Corporation> Corporations
        {
            get
            {
                if (corporations != null)
                    return corporations;
                corporations = new Dictionary<int, Corporation>();
                return corporations;
            }
        }

        public Dictionary<int, ConquerableStation> ConquerableStations
        {
            get
            {
                if (conquerables != null)
                    return conquerables;
                conquerables = new Dictionary<int, ConquerableStation>();
                return conquerables;
            }
        }

        public Dictionary<int, Alliance> Alliances
        {
            get
            {
                if (alliances != null)
                    return alliances;
                alliances = new Dictionary<int, Alliance>();
                return alliances;
            }
        }

        public Dictionary<int, Sovereignty> MapSovereignty
        {
            get
            {
                if (mapSovereignty != null)
                    return mapSovereignty;
                mapSovereignty = new Dictionary<int, Sovereignty>();
                return mapSovereignty;
            }
        }

        public Dictionary<int, Kills> MapKills
        {
            get
            {
                if (kills != null)
                    return kills;
                kills = new Dictionary<int, Kills>();
                return kills;
            }
        }

        public Dictionary<int, Jumps> MapJumps
        {
            get
            {
                if (mapJumps != null)
                    return mapJumps;
                mapJumps = new Dictionary<int, Jumps>();
                return mapJumps;
            }
        }

        public Dictionary<int, Character> Characters
        {
            get
            {
                if (characters != null)
                    return characters;
                characters = new Dictionary<int, Character>();
                return characters;
            }
        }

        public Dictionary<int, Starbase> Starbases
        {
            get
            {
                if (starbases != null)
                    return starbases;
                starbases = new Dictionary<int, Starbase>();
                return starbases;
            }
        }

        #endregion

        public void ClearAPIData()
        {
            if (DynamicStore.Instance.ApiData.ApiLastRefresh.AddHours(24) < DateTime.Now)
            {
                corporations = null;
                conquerables = null;
                alliances = null;
                mapSovereignty = null;
                kills = null;
                mapJumps = null;
                characters = null;
                starbases = null;
            }
        }

        public void UpdateAllApiData()
        {
            Alliance.UpdateAlliances();
            ConquerableStation.UpdateConquerableStations();
            Jumps.UpdateJumps();
            Kills.UpdateKills();
            Sovereignty.UpdateSovereignty();
            using (
                FileStream myStream = new FileStream(Files.EVEMapDirectory + responseCacheFilename, FileMode.Create))
            {
                ResponseCache.Save(myStream);
            }
        }

        public void UpdateMapApipData()
        {
            Jumps.UpdateJumps();
            Kills.UpdateKills();
            using (FileStream myStream = new FileStream(Files.EVEMapDirectory + responseCacheFilename, FileMode.Create))
            {
                ResponseCache.Save(myStream);
            }
        }

        public void UpdateSovereigntyApiData()
        {
            ConquerableStation.UpdateConquerableStations();
            Sovereignty.UpdateSovereignty();
            using (FileStream myStream = new FileStream(Files.EVEMapDirectory + responseCacheFilename, FileMode.Create))
            {
                ResponseCache.Save(myStream);
            }
        }

        public void UpdateAllianceApiData()
        {
            Alliance.UpdateAlliances();
            using (
                FileStream myStream = new FileStream(Files.EVEMapDirectory + responseCacheFilename, FileMode.Create))
            {
                ResponseCache.Save(myStream);
            }
        }

        public void UpdateAllStarbases()
        {
            foreach (KeyValuePair<int, Character> character in characters)
            {
                if (character.Value.Selected)
                    Data.Starbases.ImportUpdateStarbases(apiinfo, character.Value.CharacterId);
            }
        }
    }

    public enum ApiFetch
    {
        Alliance,
        Sovereignty,
        MapKills,
        Full,
        Starbase
    }
}