using System;
using libEveMapNG.Data.Dynamic;
using libEveMapNG.Data.Static;

namespace libEveMapNG.Utility
{
    public static class JumpDrive
    {
        private const double LIGHT_YEAR = 9.4605284E+15;

        public static float GetDistance(SolarSystem a, SolarSystem b)
        {
            //10e-15*1.5 is the scaling factor applied to the coordinates to draw correctly in the viewport
            //We unscale them here to find the distance between points in 3d space as described by CCP
            float xdif = a.Coordinates.X - b.Coordinates.X;
            float ydif = a.Coordinates.Y - b.Coordinates.Y;
            float zdif = a.Coordinates.Z - b.Coordinates.Z;
            float distance = (float) (Math.Sqrt(xdif*xdif + ydif*ydif + zdif*zdif)/LIGHT_YEAR);
            return distance;
        }

        public static int GetFuel(float distance, JumpRouteParameters route)
        {
            var JFs = new[] {"Rhea", "Anshar", "Ark", "Nomad"};

            int BaseConsumption = route.CapitalShipProfile.Ship.FuelConsumption;
            double JFCModifier = (1 - (0.1*route.CapitalShipProfile.JFC));
            double JFModifier = (1 - (0.1*route.CapitalShipProfile.JF));

            double BaseFuel = (BaseConsumption*distance)*JFCModifier;

            foreach (string s in JFs)
            {
                if (route.CapitalShipProfile.Ship.Name == s)
                    return (int) Math.Ceiling(BaseFuel*JFModifier);
            }
            return (int) Math.Ceiling(BaseFuel);
        }

        public static int GetFuel(float distance, ShipProfile profile)
        {
            var JFs = new[] {"Rhea", "Anshar", "Ark", "Nomad"};

            int BaseConsumption = profile.Ship.FuelConsumption;
            double JFCModifier = (1 - (0.1*profile.JFC));
            double JFModifier = (1 - (0.1*profile.JF));

            double BaseFuel = (BaseConsumption*distance)*JFCModifier;

            foreach (string s in JFs)
            {
                if (profile.Ship.Name == s)
                    return (int) Math.Ceiling(BaseFuel*JFModifier);
            }
            return (int) Math.Ceiling(BaseFuel);
        }


        public static int GetFuel(SolarSystem a, SolarSystem b, JumpRouteParameters route)
        {
            return GetFuel(GetDistance(a, b), route);
        }

        public static float GetRange(int JDC, CapitalShips.Ship shipType)
        {
            return shipType.JumpDistance + (shipType.JumpDistance*0.25f*JDC);
        }

        /* Mass is in MILLIONS of KG */

        public static int GetJumpBridgeFuel(SolarSystem a, SolarSystem b, float mass)
        {
            return (int) Math.Ceiling(mass*GetDistance(a, b)*0.5f);
        }
    }
}